// Copyright (c) 2002-2007 Telelogic AB
// All rights reserved.

/*
<trigger>
	<usage>bsf verify_associated_task_states.js states</usage>
	<version>5.0</version>
	<description>
		CR 遷移中に、関連タスクが指定した状態にあるかどうかを検査します。
		関連タスクの 1 つでも指定した状態にない場合は、このトリガーは失敗します。
		遷移前トリガーとして使用すると、トリガーの失敗によって遷移を取りやめることができます。
		例えば、CR を完了する遷移がある場合に、このトリガーを "completed" 引数付きで使用すると、遷移の前に、すべての関連タスクが "completed" 状態にあることを確認してから、遷移を行わせることができます。
</description>
	<argument1>
		<name>states</name>
		<required>no</required>
		<type>string</type>
		<description>1 つまたは空白で区切られた複数の状態名をチェックします。
		</description>
	</argument1>
</trigger>
*/

var args = bsf.lookupBean('args');
var event = bsf.lookupBean('event');
var change = bsf.lookupBean('api');
var log = bsf.lookupBean('log');

if (args.length < 1)
{
	log.error('Could not verify associated tasks states: no state was given to check.');
	log.error('Usage: verify_associated_task_states.js state [or_state...]');
}
else
{
	var expectedStates = args;
	var expectedStateSet = new java.util.HashSet(java.util.Arrays.asList(expectedStates));

	var taskDetails = change.getAssociatedTaskDetails(event.adminUser.token, event.objectId, ['status']);
	var tasks = taskDetails.tasks;
	
	// There could be multiple DCM copies of tasks in different
	// databases. We only need to confirm that at least one copy
	// of each task is in the expected state. This normally
	// only affects users that DCM working tasks around.
	// Store all task states in a map from task IDs to an array of
	// states for the task.
	var taskIdsToStates = {};

	for (var i = 0; i < tasks.length; i++)
	{
		var task = tasks[i];
		var states = taskIdsToStates[task.taskId];
		
		if (!states)
		{
			states = [];
			taskIdsToStates[task.taskId] = states;
		}

		states.push(task.getAttribute('status').stringValue);
	}
	
	for (var taskId in taskIdsToStates)
	{
		var allStates = taskIdsToStates[taskId];

		var someStateIsExpected = allStates.some(function(actualState) {
			return expectedStateSet.contains(actualState);
		});

		if (!someStateIsExpected)
		{
			var stopMessage = expectedStates.length == 1 ?
				change.getMessage('notAllTasksInStateX', [expectedStates[0]]) :
				change.getMessage('notAllTasksInStateListX', [expectedStates.join(', ')]);

			event.stopTransition(stopMessage);
			break;
		}
	}

	// The tasks we found are good, but in central server mode some task
	// databases could be offline. In that case, we can't check all the task
	// states. The offlineTasks property will contain a best guess at which
	// tasks are missing, so if it has anything, we'll be conservative and fail.
	// This may cause false negatives. In standalone mode there are never offline tasks.
	if (taskDetails.offlineTaskIds.length == 1)
	{
		event.stopTransition(change.getMessage(
			'failedToVerifyOfflineTaskX',
			[taskDetails.offlineTaskIds[0]]));
	}
	else if (taskDetails.offlineTaskIds.length > 1)
	{
		event.stopTransition(change.getMessage(
			'failedToVerifyOfflineTaskListX',
			[taskDetails.offlineTaskIds.join(', ')]));
	}
}

